'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createStore;

var _duck = require('./duck');

function createStore(reduxStore) {
  var listeners = [];

  function subscribe(listener) {
    listeners.push(listener);
    var isSubscribed = true;

    return function unsubscribe() {
      if (!isSubscribed) return;

      isSubscribed = false;
      listeners.splice(listeners.indexOf(listener), 1);
    };
  }

  function trigger(cache) {
    // Update the redux with the changes
    reduxStore.dispatch((0, _duck.update)(cache));

    // Trigger listeners to refetch possible invalidated data
    listeners.slice().forEach(function (listener) {
      return listener();
    });
    return cache;
  }

  return {
    subscribe: subscribe,
    trigger: trigger
  };
}